Releases
--------

Steps for creating a new release of YAML form module


--------------------------------------------------------------------------------

1. Convert to short array syntax
2. Tidy YAML files
3. Inspect code (PHPStorm / Coder review)
4. Run SimpleTests and UnitTests
5. Generate and edit release notes
6. Tag a release
7. Update project page
8, Create new release

--------------------------------------------------------------------------------


# Convert to short array syntax

    wget https://raw.githubusercontent.com/thomasbachem/php-short-array-syntax-converter/master/convert.php
    find . -name "*.install" -exec php "convert.php" -w "{}" \;
    find . -name "*.module" -exec php "convert.php" -w "{}" \;
    find . -name "*.php" -exec php "convert.php" -w "{}" \;
    find . -name "*.inc" -exec php "convert.php" -w "{}" \;
    rm convert.php
    
--------------------------------------------------------------------------------
    
# Tidy YAML files

    drush yamlform-tidy yamlform; drush yamlform-tidy yamlform_test;

--------------------------------------------------------------------------------

# Inspect code (PHPStorm / Coder)

## References

- [Drupal Development using PhpStorm](https://confluence.jetbrains.com/display/PhpStorm/Drupal+Development+using+PhpStorm#DrupalDevelopmentusingPhpStorm-CoderandPHPCodeSnifferIntegration)
- [Installing Coder Sniffer](https://www.drupal.org/node/1419988)

## Online

- http://git.drupal.org/project/yamlform.git 8.x-1.x
- [pareview.sh: Repeat review](http://pareview.sh/node/561406/repeat)

## [Commandline](https://www.drupal.org/node/1587138)

    # Check Drupal coding standards
    phpcs --standard=Drupal --extensions=php,module,inc,install,test,profile,theme,js,css,info,txt,md modules/sandbox/yamlform

    # Check Drupal best practices
    phpcs --standard=DrupalPractice --extensions=php,module,inc,install,test,profile,theme,js,css,info,txt,md modules/sandbox/yamlform


--------------------------------------------------------------------------------
    
# Run SimpleTest and PHPUnit

## Manual Testing

### YAML form test module (yamlform_test) 

The YAML form test module (yamlform_test) includes feature specific and working 
examples for all functionality, setting, customizations, etc...

Make sure to allow the YAML form test module to be installed.

    # Add the below setting to you settings.php file.
    $settings['extension_discovery_scan_tests'] = FALSe;

### Review TEST-SCRIPT.md

## SimpleTest

GUI (admin/config/development/testing)

Command-line (from site root)

    # Run all test included the yamlform.module 
    php core/scripts/run-tests.sh --url http://localhost/d8_dev --module yamlform
    
    # Run a single test class in the yamlform.module
    php core/scripts/run-tests.sh --verbose --url http://localhost/d8_dev --class "Drupal\yamlform\test\YamlFormAccessTest"
    
## PHPUnit tests

PHPUnit tests needs to be run individual if this module is place in a 
modules sub-directory (ie module/contrib or modules/sandbox).
 
- [/core/phpunit.xml.dist only finds top-level contrib modules](https://www.drupal.org/node/2499239)

If the above patch is applied or this module is in modules/yamlform. 

This command will execute all PHPUnit tests for the YamlForm module

    cd core
    php ../vendor/phpunit/phpunit/phpunit --group YamlFormUnit

The below commands can be used to execute each individual PHPUnit test.

    cd core
    export SIMPLETEST_DB=mysql://drupal_d8_dev:drupal.@dm1n@localhost/drupal_d8_dev;
    php ../vendor/phpunit/phpunit/phpunit ../modules/sandbox/yamlform/tests/src/Unit/YamlFormTidyTest.php
    php ../vendor/phpunit/phpunit/phpunit ../modules/sandbox/yamlform/tests/src/Unit/YamlFormHelperTest.php
    php ../vendor/phpunit/phpunit/phpunit ../modules/sandbox/yamlform/tests/src/Unit/YamlFormOptionsHelperTest.php
    php ../vendor/phpunit/phpunit/phpunit ../modules/sandbox/yamlform/tests/src/Unit/YamlFormArrayHelperTest.php     
    php ../vendor/phpunit/phpunit/phpunit ../modules/sandbox/yamlform/src/Tests/YamlFormEntityInputsValidationUnitTest.php    
    
--------------------------------------------------------------------------------

# Generate and edit release notes

Git Release Notes for Drush (https://www.drupal.org/project/grn)

    drush release-notes 8.x-1.0-unstable5 8.x-1.x
    
--------------------------------------------------------------------------------
    
# Tag a release

    git tag 8.x-1.0-unstable6
    git push --tags
    git push origin tag 8.x-1.0-unstable6

--------------------------------------------------------------------------------

# Update project page

Converting README.md to HTML for Drupal.org

- [Convert Markdown to HTML](http://daringfireball.net/projects/markdown/dingus)
- [Tidy HTML](https://infohound.net/tidy/)
    - No indent
    - No wrap   
- Manually cleanup
    - Remove `<html>` tags.
    - Remove 'Contents of this file'.
    - Update example code and wrap in `<pre>` tag.
- [Update project page](https://www.drupal.org/node/2640714/edit)
    - Preview 
    - Save

#  Create new release

- [Create Release](https://www.drupal.org/node/add/project-release/2640714)
- [Administer releases](https://www.drupal.org/node/2640714/edit/releases)
