<?php

/**
 * @file
 * Contains \Drupal\yamlform\Plugin\YamlFormElement\OptionsBase.
 */

namespace Drupal\yamlform\Plugin\YamlFormElement;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\OptGroup;
use Drupal\yamlform\Utility\YamlFormOptionsHelper;
use Drupal\yamlform\YamlFormElementBase;
use Drupal\yamlform\YamlFormSubmissionInterface;

/**
 * Provides a base 'options' element.
 */
abstract class OptionsBase extends YamlFormElementBase {

  /**
   * {@inheritdoc}
   */
  public function prepare(array &$element, YamlFormSubmissionInterface $yamlform_submission) {
    $element['#element_validate'][] = [get_class($this), 'validate'];
  }

  /**
   * {@inheritdoc}
   */
  public function formatText(array &$element, $value) {
    // Return empty value.
    if ($value === '' || $value === NULL) {
      return '';
    }

    $flattened_options = OptGroup::flattenOptions($element['#options']);
    if (is_array($value)) {
      return implode(', ', YamlFormOptionsHelper::getOptionsText($value, $flattened_options));
    }
    else {
      return YamlFormOptionsHelper::getOptionText($value, $flattened_options);
    }
  }

  /**
   * Form API callback. Remove unchecked options from value array.
   */
  public static function validate(array &$element, FormStateInterface $form_state) {
    $name = $element['#name'];
    $values = $form_state->getValue($name);
    $values = array_filter($values);
    $form_state->setValue($name, $values);
  }

}
