<?php

/**
 * @file
 * Provides Drupal\yamlform\YamlFormElementBase.
 */

namespace Drupal\yamlform;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Provides a base class for a YAML form element.
 *
 * @see \Drupal\yamlform\YamlFormElementInterface
 * @see \Drupal\yamlform\YamlFormElementManager
 * @see plugin_api
 */
class YamlFormElementBase extends PluginBase implements YamlFormElementInterface {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function label() {
    return $this->pluginDefinition['label'];
  }

  /**
   * {@inheritdoc}
   */
  public function description() {
    return $this->pluginDefinition['description'];
  }

  /**
   * {@inheritdoc}
   */
  public function isMultiline() {
    return $this->pluginDefinition['multiline'];
  }

  /**
   * {@inheritdoc}
   */
  public function prepare(array &$element, YamlFormSubmissionInterface $yamlform_submission) {}

  /**
   * {@inheritdoc}
   */
  public function setDefaultValue(array &$element, $default_value) {}

  /**
   * {@inheritdoc}
   */
  public function save(array &$element, YamlFormSubmissionInterface $yamlform_submission) {}

  /**
   * {@inheritdoc}
   */
  public function postDelete(array &$element, YamlFormSubmissionInterface $yamlform_submission) {}

  /**
   * {@inheritdoc}
   */
  public function buildHtml(array &$element, $value) {
    $build = [];
    $build['#type'] = 'item';
    if (isset($element['#title'])) {
      $build['#title'] = $element['#title'];
    }
    $build['#name'] = $element['#key'];
    $build['#markup'] = $this->formatHtml($element, $value);
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function buildText(array &$element, $value) {
    $value = $this->formatText($element, $value);
    if ($element['#title']) {
      return [
        '#markup' => $element['#title'] . ($this->isMultiline() ? ":\n" : ": ") . $value,
        '#suffix' => "\n",
      ];
    }
    else {
      return [
        '#markup' => $value,
        '#suffix' => "\n",
      ];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function formatHtml(array &$element, $value) {
    return $this->formatText($element, $value);
  }

  /**
   * {@inheritdoc}
   */
  public function formatText(array &$element, $value) {
    // Return empty value.
    if ($value === '' || $value === NULL) {
      return '';
    }

    // Format value based on the element type using default settings.
    if (isset($element['#type'])) {
      // Apply #field prefix and #field_suffix to value.
      if (isset($element['#field_prefix'])) {
        $value = $element['#field_prefix'] . $value;
      }
      if (isset($element['#field_suffix'])) {
        $value .= $element['#field_suffix'];
      }
    }

    return $value;
  }

}
